@@ -133,13 +133,12 @@ def trim_internal(x, axes, boundary=None):
                 else:
                     d = d - overlap * 2
 
+            elif isinstance(overlap, tuple):
+                d = d - overlap[0] if j != 0 else d
+                d = d - overlap[1] if j != len(bd) - 1 else d
             else:
-                if isinstance(overlap, tuple):
-                    d = d - overlap[0] if j != 0 else d
-                    d = d - overlap[1] if j != len(bd) - 1 else d
-                else:
-                    d = d - overlap if j != 0 else d
-                    d = d - overlap if j != len(bd) - 1 else d
+                d = d - overlap if j != 0 else d
+                d = d - overlap if j != len(bd) - 1 else d
 
             ilist.append(d)
         olist.append(tuple(ilist))
@@ -307,9 +306,9 @@ def boundaries(x, depth=None, kind=None):
     constant
     """
     if not isinstance(kind, dict):
-        kind = {i: kind for i in range(x.ndim)}
+        kind = dict.fromkeys(range(x.ndim), kind)
     if not isinstance(depth, dict):
-        depth = {i: depth for i in range(x.ndim)}
+        depth = dict.fromkeys(range(x.ndim), depth)
 
     for i in range(x.ndim):
         d = depth.get(i, 0)
@@ -457,7 +456,7 @@ def overlap(x, depth, boundary, *, allow_rechunk=True):
         )  # this is a no-op if x.chunks == new_chunks
 
     else:
-        original_chunks_too_small = any([min(c) < d for d, c in zip(depths, x.chunks)])
+        original_chunks_too_small = any(min(c) < d for d, c in zip(depths, x.chunks))
         if original_chunks_too_small:
             raise ValueError(
                 "Overlap depth is larger than smallest chunksize.\n"
@@ -725,7 +724,7 @@ def map_overlap(
         args = [new_collection(a) for a in args]
 
     # Escape to map_blocks if depth is zero (a more efficient computation)
-    if all([all(depth_val == 0 for depth_val in d.values()) for d in depth]):
+    if all(all(depth_val == 0 for depth_val in d.values()) for d in depth):
         return map_blocks(func, *args, **kwargs)
 
     for i, x in enumerate(args):
