@@ -178,7 +178,10 @@ def test_open_glob(dir_server):
 def test_parquet(engine):
     pytest.importorskip("requests", minversion="2.21.0")
     dd = pytest.importorskip("dask.dataframe")
-    pytest.importorskip(engine)
+    pa = pytest.importorskip(engine)
+    if Version(pa.__version__) >= Version("22.0.0"):
+        pytest.skip(reason="https://github.com/apache/arrow/issues/47981")
+
     df = dd.read_parquet(
         [
             "https://github.com/Parquet/parquet-compatibility/raw/"
