@@ -621,15 +621,14 @@ class ArrowDatasetEngine(Engine):
                 # User does not want to set index and an
                 # "unwanted" column has been set to the index
                 df.reset_index(drop=True, inplace=True)
-        else:
-            if set(df.index.names) != set(index) and index_in_columns_and_parts:
-                # The wrong index has been set and it contains
-                # one or more desired columns/partitions
-                df.reset_index(drop=False, inplace=True)
-            elif index_in_columns_and_parts:
-                # The correct index has already been set
-                index = False
-                columns_and_parts = list(set(columns_and_parts) - set(df.index.names))
+        elif set(df.index.names) != set(index) and index_in_columns_and_parts:
+            # The wrong index has been set and it contains
+            # one or more desired columns/partitions
+            df.reset_index(drop=False, inplace=True)
+        elif index_in_columns_and_parts:
+            # The correct index has already been set
+            index = False
+            columns_and_parts = list(set(columns_and_parts) - set(df.index.names))
         df = df[list(columns_and_parts)]
 
         if index:
@@ -1573,11 +1572,10 @@ class ArrowDatasetEngine(Engine):
                                 else:
                                     cstats += [cmin, cmax, null_count]
                                 cmax_last[name] = cmax
+                            elif single_rg_parts:
+                                s["columns"].append({"name": name})
                             else:
-                                if single_rg_parts:
-                                    s["columns"].append({"name": name})
-                                else:
-                                    cstats += [None, None, None]
+                                cstats += [None, None, None]
                         if gather_statistics:
                             file_row_group_stats[fpath].append(s)
                             if not single_rg_parts:
