@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import codecs
 import functools
+import gc
 import inspect
 import os
 import re
@@ -12,7 +13,7 @@ import types
 import uuid
 import warnings
 from collections.abc import Callable, Hashable, Iterable, Iterator, Mapping, Set
-from contextlib import contextmanager, nullcontext, suppress
+from contextlib import ContextDecorator, contextmanager, nullcontext, suppress
 from datetime import datetime, timedelta
 from errno import ENOENT
 from functools import wraps
@@ -742,9 +743,14 @@ class Dispatch:
     def dispatch(self, cls):
         """Return the function implementation for the given ``cls``"""
         lk = self._lookup
+        if cls in lk:
+            return lk[cls]
         for cls2 in cls.__mro__:
             # Is a lazy registration function present?
-            toplevel, _, _ = cls2.__module__.partition(".")
+            try:
+                toplevel, _, _ = cls2.__module__.partition(".")
+            except Exception:
+                continue
             try:
                 register = self._lazy[toplevel]
             except KeyError:
@@ -752,7 +758,10 @@ class Dispatch:
             else:
                 register()
                 self._lazy.pop(toplevel, None)
-                return self.dispatch(cls)  # recurse
+                meth = self.dispatch(cls)  # recurse
+                lk[cls] = meth
+                lk[cls2] = meth
+                return meth
             try:
                 impl = lk[cls2]
             except KeyError:
@@ -1180,7 +1189,7 @@ def asciitable(columns, rows):
     """
     rows = [tuple(str(i) for i in r) for r in rows]
     columns = tuple(str(i) for i in columns)
-    widths = tuple(max(max(map(len, x)), len(c)) for x, c in zip(zip(*rows), columns))
+    widths = tuple(max(*map(len, x), len(c)) for x, c in zip(zip(*rows), columns))
     row_template = ("|" + (" %%-%ds |" * len(columns))) % widths
     header = row_template % tuple(columns)
     bar = "+%s+" % "+".join("-" * (w + 2) for w in widths)
@@ -1750,8 +1759,8 @@ def format_time_ago(n: datetime) -> str:
         "minutes": lambda diff: diff.seconds % 3600 / 60,
     }
     diff = datetime.now() - n
-    for unit in units:
-        dur = int(units[unit](diff))
+    for unit, func in units.items():
+        dur = int(func(diff))
         if dur > 0:
             if dur == 1:  # De-pluralize
                 unit = unit[:-1]
@@ -2033,23 +2042,6 @@ def stringify(obj, exclusive: Iterable | None = None):
     elif exclusive is None:
         return str(obj)
 
-    if typ is tuple and obj:
-        from dask.optimization import SubgraphCallable
-
-        obj0 = obj[0]
-        if type(obj0) is SubgraphCallable:
-            obj0 = obj0
-            return (
-                SubgraphCallable(
-                    stringify(obj0.dsk, exclusive),
-                    obj0.outkey,
-                    stringify(obj0.inkeys, exclusive),
-                    obj0.name,
-                ),
-            ) + tuple(stringify(x, exclusive) for x in obj[1:])
-        elif callable(obj0):
-            return (obj0,) + tuple(stringify(x, exclusive) for x in obj[1:])
-
     if typ is list:
         return [stringify(v, exclusive) for v in obj]
     if typ is dict:
@@ -2102,6 +2094,37 @@ def _cumsum(seq, initial_zero):
         return tuple(toolz.accumulate(add, seq))
 
 
+@functools.lru_cache
+def _max(seq):
+    if isinstance(seq, _HashIdWrapper):
+        seq = seq.wrapped
+    return max(seq)
+
+
+def cached_max(seq):
+    """Compute max with caching.
+
+    Caching is by the identity of `seq` rather than the value. It is thus
+    important that `seq` is a tuple of immutable objects, and this function
+    is intended for use where `seq` is a value that will persist (generally
+    block sizes).
+
+    Parameters
+    ----------
+    seq : tuple
+        Values to reduce
+
+    Returns
+    -------
+    tuple
+    """
+    assert isinstance(seq, tuple)
+    # Look up by identity first, to avoid a linear-time __hash__
+    # if we've seen this tuple object before.
+    result = _max(_HashIdWrapper(seq))
+    return result
+
+
 def cached_cumsum(seq, initial_zero=False):
     """Compute :meth:`toolz.accumulate` with caching.
 
@@ -2173,8 +2196,6 @@ def show_versions() -> None:
 
     stdout.writelines(dumps(result, indent=2))
 
-    return
-
 
 def maybe_pluralize(count, noun, plural_form=None):
     """Pluralize a count-noun string pattern when necessary"""
@@ -2277,3 +2298,28 @@ class shorten_traceback:
         #     return exc_tb.tb_next
 
         return exc_tb
+
+
+def unzip(ls, nout):
+    """Unzip a list of lists into ``nout`` outputs."""
+    out = list(zip(*ls))
+    if not out:
+        out = [()] * nout
+    return out
+
+
+class disable_gc(ContextDecorator):
+    """Context manager to disable garbage collection."""
+
+    def __init__(self, collect=False):
+        self.collect = collect
+        self._gc_enabled = gc.isenabled()
+
+    def __enter__(self):
+        gc.disable()
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if self._gc_enabled:
+            gc.enable()
+        return False
